//// Do not copy four slash comments, they're for you to read.
//// Add the following function to ExtParserFunctions class:

	// Return a three column html table of subpage links rooted at the
	// parent's page. This allows navigating down path-names to provide
	// virtical tree browsing within a namespace, and horizontal jumping
	// between similarly named trees in other namespaces.
	// Note: This only lists the parent's immediate children.
	// FIXME: the column count should be a parametre, not hard-coded
	// syntax: {{#subpages: [NAMESPACE]}}}
	// 	where: NAMESPACE = name, index, or null
	function subpages( &$parser, $NS = NULL ) {
		global $wgCanonicalNamespaceNames; // for validating index
		$page_tab = '<span style="color:darkred;font-weight:800">
		[subpages] Warning: invalid namespace "'.$NS.'"</span>';

		// figure out if $NS is name, number, or null
		$ns_parent = $parser->getTitle()->getNamespace();
		if ( !is_numeric( $NS ))
			if ( !empty( $NS ))
				$ns_child = Namespace::getCanonicalIndex( strtolower( $NS ) );
			else $ns_child = $ns_parent;
		else $ns_child = intval( $NS );

		// validate ns and set caption
		if ( NS_MAIN != $ns_child ) // 0 is valid, but not in ns array
			if ( is_null( $ns_child )
					|| !array_key_exists( $ns_child, $wgCanonicalNamespaceNames ))
				return $page_tab; // invalid ns
			else $ns_caption = Namespace::getCanonicalName( $ns_child );
		else $ns_caption = "Article";
		$subpage = $ns_child == $ns_parent; // false == not a subpage

		// select parent's title + one path name
		$title_parent = str_replace( " ","_", $parser->getTitle()->getText() );
		$dbr = wfGetDB( DB_SLAVE );
		// DISTINCT ON may be postgresql specific, change for other databases.
		$res = $dbr->select(
			array( 'page' ),
			array( 'DISTINCT ON(page_title,page_namespace) page_title'/*, 'page_namespace'*/ ),
			array(
				'page_title ~ \'^'.$title_parent.'/[^/]+$\'',
				'page_namespace =  \''.$ns_child.'\'',
				'page_is_redirect = \'0\''),'',
			array( 'ORDER BY' => 'page_title ASC')/*,
			__METHOD__*/);
		$rows = $dbr->numRows( $res );
		// setup subpage table with one row and three columns
		if ( 0 < $rows ) {
			$limit = $rows / 3; // three columns per row
			$page_tab = '
<table width="100%">
	<caption style="font-size:14pt">'
		.$ns_caption.' Sub-pages ('.$rows.')</caption>
	<tr>';
		// set link as either subpage, namespace, or main
			$link_prefix = '
			<li>[[';
			if ( !$subpage )
				if ( NS_MAIN != $ns_child )
					$link_prefix .= ':'.$ns_caption. ':' .$title_parent.'/';
				else $link_prefix .= $title_parent.'/';
			else $link_prefix .= '/';
			$link_suffix = ']]</li>';

			// names ordered by column and then row
			for( $col = 0; 3 > $col; ++$col ) {
				$page_tab .= '
		<td valign="top">';
				$ul = false;
			// NOTE: the list may have less than $limit objects
			// NOTE: $row is the list index, not the db row
				for( $row = 0; $limit > $row; ++$row ) {
					$x =& $dbr->fetchObject( $res );
					if ( $x ) {
						if ( !$ul ) {// open list
							$page_tab .= "<ul>";
							$ul = true; // remember to close list
						}
					$title_path = explode( '/', $x->page_title );
					$path_count = count( $title_path );
			// if the title is a pathed name, select the rightmost name
					if ( 0 < $path_count )
						$page_name = $title_path[ $path_count - 1 ];
					else
						$page_name = $x->page_title;
			// this is correct: the name must appear twice;
			// 1st for ending the link, 2nd for displayed link name
					if ( !$subpage )
						$page_name .= '|'.$page_name;
					$page_tab .= $link_prefix.$page_name.$link_suffix;
					} else
					{
				if ( $ul ) // open list, close it
					$page_tab .= "
		</ul>";
				$page_tab .= "</td>";
					 break 2; // short list, simply continue
					}
				}
				if ( $ul ) // open list, close it
					$page_tab .= "
		</ul>";
				$page_tab .= "</td>";
			}
			$page_tab .= '
	</tr>
</table>';
		} else $page_tab = ''; // no matches, ignore
		$dbr->freeResult( $res );
		return $page_tab;
	}

//// Add the following to wfSetupParserFunctions():

	$wgParser->setFunctionHook( 'subpages', array( &$wgExtParserFunctions, 'subpages' ));

//// Add the following in the DEFAULT case switch of wfParserFunctionsLanguageGetMagic()

			$magicWords['subpages']= array( 0, 'subpages' );
